//  conversor.cpp: definiciones para el programa 22
//  Fecha: 9/5/2020; 12 m
//  Revisin: 13/9/2020; 5:50 am
//  Por: S. Galliano.
#include "conversor.h"
#include <iostream>

// constructor
Cargador::Cargador() {
    id = "c:/temp/BD/equivalencias.txt";
}

#include <cstdlib>
#include <cstring>

//  carga la base de datos c:/temp/BD/equivalencias.txt
bool Cargador::leerData( std::istream & leer, Vct & vect ) {
    if ( leer.fail() )
        return false;

    //  leer todos sus registros
    char * buf;
    char * val[4]; // cuatro valores de caracteres
    const char * delim = ",";
    Str linea, tp, ue, um, k; // tipo, uEn, uMet, Konst

    while ( leer.good() ) {
        //  construyendo un registro
        getline( leer, linea );                    // lee una lnea
        buf = const_cast<char *>( linea.c_str() ); // la pone en una cadena C
        char * tokenPtr = strtok( buf, delim );    // fija el primer token

        for ( short i = 0; tokenPtr not_eq nullptr; ++i ) {
            val[i] = tokenPtr;                     // guarda el token previo...
            tokenPtr = strtok( nullptr, delim );   // ...y busca el siguiente
        } // for

        // pasa los cuatro tokens del registro a las variables
        tp = val[0];
        ue = val[1];
        um = val[2];
        k  = val[3];

        Dato tmp = std::make_tuple( tp, ue, um, atof( k.c_str() ) );

        //  inserta el nodo en la lista
        vect.push_back( tmp );
    } // while

    return true;
}

#include <cmath>

void Cargador::convertir( double val ) {
    Str tipo, uEn, uMt;
    double vEn, vMt, k;

    // sacar los datos
    std::tie( tipo, uEn, uMt, k ) = dat;

    if ( "TP" == tipo ) { // temperaturas
        vEn = 1.8 * val + k;
        vMt = ( val - k ) / 1.8;
        inf = make_tuple( val, vEn, vMt, uEn, uMt );
    } else if ( "AN" == tipo ) { // ngulos
        vMt = val * k;

        if ( vMt > 360 ) vMt = fmod( vMt, 360 );

        inf = make_tuple( val, val / k, vMt, uEn, uMt );
    } else if ( "DM" == tipo ) { // diabetes
        vEn = ( val * k ) + 3.75;
        vMt = ( val - 3.75 ) / k;
        inf = make_tuple( val, vEn, vMt, uEn, uMt );
    } else {
        // empacar la informacin
        inf = make_tuple( val, val / k, val * k, uEn, uMt );
    } // if-en-cascada
}

#include <iostream>
#include <iomanip>

void Cargador::imprimir() {
    double
    val,       // el valor a convertir
    valEN,     // valor convertido a ingls
    valMT;     // valor convertido a mtrico
    Str uniEN, // unidad inglesa
        uniMT; // unidad mtrica

    // desempacando la informacin
    tie( val, valEN, valMT, uniEN, uniMT ) = inf;

    //  la salida discriminada
    std::ios::fmtflags oldFlags = std::cout.flags(); // formato original

    if ( uniEN == "mmol/l" ) {
        // primera lnea de diabetes
        std::cout << '\n'  <<
                  val << " " << uniEN << " equivalen a " << std::fixed
                  << std::setprecision( 1 ) << valMT << " " << uniMT << "\n\n";
    } else {
        std::cout << setiosflags( std::ios::fixed | std::ios::showpoint )
                  << std::left << std::setprecision( 4 );
        // primera lnea en general
        std::cout << '\n'
                  << std::setw( 4 ) << val   << " " << std::setw( 6 ) << uniEN
                  << "--> " << std::setw( 8 ) << valMT << " " << uniMT;
        // segunda lnea va a consola si no es diabetes
        std::cout << '\n'
                  << std::setw( 4 ) << val   << " " << std::setw( 6 ) << uniMT
                  << "--> " << std::setw( 8 ) << valEN << " " << uniEN << "\n\n";
    } // if-else

    std::cout.flags( oldFlags ); // formato original restaurado
}
