//  grafica.cpp: definiciones para el programa
//  Fecha: 3/3/2020; 4:42 pm
//  Revisin: 13/8/2020; 6:17 am
//  Por: S. Galliano.
#include "grafica.h"
#include <fstream>
#include <cstdlib>
#include <cstring>
using std::strtok;
using std::ifstream;

/*
    Estos son los tokens:

    129 mg/dl equivalen a  7.2 mmol/l; Fri Jan 03 06:11:48 2020

    0) 129
    1) mg/dl
    2) equivalen
    3) a
 -->4) 7.2
    5) mmol/ml
    6) Fri
 -->7) Jan
 -->8) 03
    9) 06:11:48
   10) 2020
*/

void cargar( Str bd, Dato & dat ) {
    ifstream Leyendo( bd );
    if ( not Leyendo ) throw ERROR;

    //  leer todos sus registros
    char * buf;
    char * val;
    const char * delim = " ";
    Str linea;
    float gluc;
    int dia;
    Str mes;

    while ( Leyendo.good() ) {
        getline( Leyendo, linea );                 // lee una lnea
        if ( linea.empty() ) continue;             // no procesa lneas vacas
        buf = const_cast<char *>( linea.c_str() ); // la pone en una cadena ANSI C
        char * tokenPtr = strtok( buf, delim );    // fija el primer token

        for ( short i = 0; tokenPtr not_eq nullptr; ++i ) {
            val = tokenPtr;                     // toma el valor
            // guarda los tres tokens de inters
            if ( 4 == i ) gluc = atof( val );   // la glucosa en mmol/ml
            if ( 7 == i ) mes = val;            // el mes de lectura
            if ( 8 == i ) dia = atoi( val );    // el da de lectura
            tokenPtr = strtok( nullptr, delim ); // busca la prxima lnea
        } // for

        dat.push_back( make_tuple( gluc, dia, mes ) ); // entra la tupla al vector
    } // while
}

#include <iostream>
using std::cout;
#include <iomanip>
using std::setw;
//  una funcion auxiliar para cargar la escala
void cargaEscala() {
    cout << std::setw( 8 ) << '|';

    for ( int i = 5; i <= 15; i += 5 )
        cout << std::setw( 5 ) << i;

    cout << " mmol/ml\n";
}

//  una funcion auxiliar para cargar el eje horizontal
void cargaEje() {
    cout << std::setw( 8 );

    for ( int i = 0; i <= 15; ++i )
        0 == i % 5 ? cout << '+' : cout << '-';

    cout << "--\n";
}

#include <algorithm>
void graficar( const Dato & dat ) {
    cout << "Valores de glucosa en sangre.\n";
    cargaEscala();
    cargaEje();

    //  los valores de la tupla
    float val;
    int dia;
    Str mes;

    for ( auto x : dat ) {
        tie( val, dia, mes ) = x;                        // saca la tupla
        cout << mes << setw( 3 ) << dia << " |";         // pone el pie del eje
        cout << setw( 2 + val ) << "*  " << val << '\n'; // pone el valor
    } // for

    cout << std::setw( 10 ) << "|\n\n"; // cierra el eje
}
