//  almacen.cpp: definiciones para el problema 25
//  Fecha: 7/4/2020; 11 pm
//  Revisin: 15/10/2020; 7 pm
//  Por: S. Galliano.
#include "almacen.h"

Almacen::Almacen() {
    adicionar( "Mocha Munch", 208, 0.9 );
    adicionar( "Krispy Krunch", 411, 1.05 );
    adicionar( "Honey Beez", 789, 1.25 );
}

//  aade un dulce al inventario
void Almacen::adicionar( Str mk, int ct, double pr ) {
    Trio temp( mk, pr, ct ); // usa el constructor
    stock.push_back( temp );
}

//  elimina un dulce por su marca
void Almacen::eliminarSnack( Str k ) {
    Stock temp;

    for ( auto x: stock ) {       //  se visita el vector
        tie( mk, pr, ct ) = x;    // se busca la clave
        if ( k == mk ) continue;  // si la clave est, se obvia
        else temp.push_back( x ); // se encola en el vector temporal
    } // ranged-for

    temp.swap( stock ); // al final se intercambian ambos vectores
}

//  comprueba si una marca est en la BD
bool Almacen::marcaEnStock( Str k ) {
    for ( auto x: stock ) {         // se visita el vector
        tie( mk, pr, ct ) = x;      // se busca la clave
        if ( k == mk ) return true; // si est regresa true
    } // ranged-for

    return false; // si no est regresa false
}

//  cambia la cantidad del dulce en inventario
void Almacen::editarCt( Str k, int c ) {
    Trio temp; // un nodo temporal
    Stock alt; // un vector alternativo

    for ( auto x: stock ) {    // se visita el vector
        tie( mk, pr, ct ) = x; // se busca la clave

        if ( k == mk ) {
            temp = std::make_tuple( mk, c, pr ); // se crea un nodo
            alt.push_back( temp );               // se encola
            continue;                            // se evita el nodo obsoleto
        } // if

        alt.push_back( x );  // se encola en el vector temporal
    } // ranged-for

    alt.swap( stock ); // al final se intercambian
}

//  cambia el precio del dulce
void Almacen::editarPr( Str k, double p ) {
    Trio temp; // un nodo temporal
    Stock alt; // un vector alternativo

    for ( auto x : stock ) {   // se visita el vector
        tie( mk, pr, ct ) = x; // se busca la clave

        if ( k == mk ) {
            temp = std::make_tuple( mk, ct, p ); // se crea un nodo
            alt.push_back( temp );               // se encola
            continue;                            // se evita el nodo obsoleto
        } // if

        alt.push_back( x ); // se encola en el vector temporal
    } // ranged-for

    alt.swap( stock ); // al final se intercambian
}

#include <iostream>
#include <ios>
#include <iomanip>
using std::cout;
using std::setw;
using std::setfill;

//  funcin auxilar: pone una lnea doble
//  o simple, de 40 caracteres en consola
void ponerLinea( char ch ) {
    cout << setfill( ch ) << setw( 40 ) << '\n' << setfill( ' ' );
}
 using std::begin;
 using std::end;

void Almacen::contarStock() {
    int totCnt = 0, totStk = 0;
    double totPre = 0.0;

    for ( auto x: stock ) {    // se visita el vector
        tie( mk, pr, ct ) = x; // se desempacan sus valores
        totCnt += ct;          // se suman sus cantidades
        totPre += ct * pr;
        ++totStk;
    } // ranged-for

    cout << "\nHay " << totStk << " productos diferentes.";
    cout << "\nHay " << totCnt << " unidades en inventario.";
    std::ios_base::fmtflags oldFlags = cout.flags();
    cout << std::fixed << std::setprecision( 2 );
    cout << "\nEl inventario vale $" << totPre << "\n\n";
    cout.flags( oldFlags );
}

//  el inventario a consola
void Almacen::mostrarStock() {
    Dulce temp;
    std::ios::fmtflags oldFlags = cout.flags();
    cout << "\nEn almac\202n:\n";
    ponerLinea( '=' );
    cout << "Marca\t\tPrecio\tStock\tCosto\n";
    ponerLinea( '-' );
    cout << std::fixed << std::setprecision( 2 );

    for ( auto x: stock ) {                         // se visita el vector
        tie( mk, pr, ct ) = x;                      // se desempacan sus valores
        temp.dulce = std::make_tuple( mk, pr, ct ); // se crea un nodo
        temp.mostrarDato();                         // se muestra en consola
    } // ranged-for

    ponerLinea( '=' );
    contarStock();
    cout.flags( oldFlags );
}
