//  entrada.cpp: definiciones para el programa 16
//  Fecha: 26/8/2019; 7:52 am
//  Revisin: 11/9/2020; 12:22 pm
//  Por S. Galliano.

#include "entrada.h"
#include <iomanip>
#include <algorithm>

double Estads::suma () {
    return accumulate( vect.begin(), vect.end(), 0.0 );
}
double Estads::promedio() {
    return suma() / vect.size();
}
double Estads::desviacion() {
    return sqrt( varianza() );
}
double Estads::mi_max() {
    return *max_element( vect.begin(), vect.end() );
}
double Estads::mi_min() {
    return *min_element( vect.begin(), vect.end() );
}
double Estads::rango() {
    return mi_max() - mi_min();
}
int Estads::cuenta() {
    return vect.size();
}

double Estads::mediana() {
    double med;
    int n = vect.size();
    int idx = 0.5 * n; // se busc el ndice del medio
    n % 2              // se calcul la mediana
        ? med = vect[idx]
        : med = 0.5 * ( vect[idx - 1] + vect[idx] );
    return med;
}

double Estads::moda() {
    unsigned cuenta; // cuenta la posible moda
    unsigned idx   = 0;
    double laModa  = 0;
    double laRacha = 0;
    unsigned n     = vect.size();

    do {
        cuenta = count( vect.begin(), vect.end(), vect[idx] );

        if ( cuenta > laRacha ) { // diferencia mayor que racha?
            laRacha = cuenta;     // ponla en la racha y...
            laModa  = vect[idx];  // ...pon el valor en la moda
            idx += cuenta;        // avanza a saltos;
        } else ++idx;             // avanza dentro de un valor repetido
    } while ( idx not_eq n ); // do-while

    if ( laRacha <= 1 ) laModa = HUGE_VAL; // no hay moda
    return laModa;
}

double Estads::varianza() {
    double total = 0;
    double ave = promedio();
    for ( auto x : vect ) total += pow( x - ave, 2 ); // se visitan los nodos
    return total / ( vect.size() - 1 );               // varianza muestral
}

#include <fstream>

//  valores ledos desde el archivo
void Estads::leerArchivo( const Str archy ) {
    // aqu hace falta cdigo defensivo, porque
    // si el fichero no est, no puede abrirlo
    std::ifstream Leyendo;
    Leyendo.open( archy );

    if ( Leyendo.is_open() ) {
        vect.clear();
        double val;
        while ( Leyendo >> val ) vect.push_back( val );
        sort( vect.begin(), vect.end() );
    } else {
        throw ERR;
    } // if-else
}

void muestra( std::ostream & escribe, Str ID, double stat ) {
    escribe.setf( std::ios::fixed );
    escribe << std::setprecision( 4 );

    if ( HUGE_VAL == stat ) {
        escribe << std::setw( 12 ) << ID << " #N/A\n";
    } else {
        escribe << std::setw( 12 ) << ID << std::setw( 8 ) << stat << '\n';
    } // if-else

    escribe.unsetf( std::ios::fixed );
    escribe << std::setprecision( 6 );
}
