//  Programa 30b: high quality random numbers
//  Halterman, ps. 403-404
//  Fecha: 2/4/2020; 3 pm
//  Revisin: 13/9/2020; 1:08 pm
//  S. Galliano.

#include <iostream>
#include <iomanip>
#include <random>

const int NUMBER_OF_TRIALS = 10;

int main() {
    std::cout << "Listing 13.11: high quality random numbers.\n\n";
    std::random_device rdev; // Used to establish a seed value

    // The timing
    clock_t totalBeg, totalEnd;
    double	totalTime = 0;
    totalBeg = clock();

    // Create a Mersenne Twister random number generator with a seed
    // value derived from rdev
    std::mt19937 mt( rdev() );

    // Create a uniform distribution object. Given a random
    // number generator, dist will produce a value in the range
    // 0...9999.
    std::uniform_int_distribution<int> dist( 0, 9999 );

    // Total counts over all the runs.
    // Make these double-precision floating-point numbers
    // so the average computation at the end will use floating-point
    // arithmetic.
    double total5 = 0.0, total9995 = 0.0;

    // Accumulate the results of 10 trials, with each trial
    // generating 10'000,000 pseudorandom numbers
    int r, count5, count9995;

    for ( int trial = 1; trial <= NUMBER_OF_TRIALS; trial++ ) {
        // Initialize counts for this run of a billion trials
        count5 = 0;
        count9995 = 0;

        // Generate 0'000,000 pseudorandom numbers in the range
        // 0...9,999 and count the number of times 5 and 9,995 appear
        for ( int i = 0; i < 10000000; i++ ) {
            // generate a pseudorandom number in the range 0...9,999
            r = dist( mt );

            if ( r == 5 )
                count5++;    // Number 5 generated, so count it
            else if ( r == 9995 )
                count9995++; // Number 9,995 generated, so count it
        } // for

        // Display the number of times the program generated 5 and 9,995
        std::cout << "Trial #" << std::setw( 2 ) << trial
                  << " 5: " << std::setw( 6 ) << count5
                  << "\t9995: " << std::setw( 6 ) << count9995 << '\n';

        total5 += count5;       // Accumulate the counts to
        total9995 += count9995; // average them at the end
    } // for

    totalEnd = clock();
    totalTime = difftime( totalEnd, totalBeg ) * 0.001;

    std::cout << "-------------------" << '\n';
    std::cout << "Averages for " << NUMBER_OF_TRIALS << " trials: 5: "
              << std::setw( 6 ) << total5 / NUMBER_OF_TRIALS << "; 9995: "
              << std::setw( 6 ) << total9995 / NUMBER_OF_TRIALS << "\n"
              << "Time: " << totalTime << " s\n\n";

    system( "pause" );
    return EXIT_SUCCESS;
}
