#ifndef PRINT_H_INCLUDED
#define PRINT_H_INCLUDED

//  print.h: declaraciones para las plantillas de impresin
//  Fecha: 21/8/2019; 4:21 pm
//  Revisin: 2/5/2020; 11:50 pm
//  Por S. Galliano.

#include <iostream>
#include <iomanip>

/**
@brief LISTA: imprime opcionalmente una cadena C++ seguida
por todos los elementos del contenedor K, separados por
comas, removiendo la coma final. Galliano, 2019.
@param <b>K:</b> contenedor del tipo T, que itere hacia adelante.
@param <b>st:</b> encabezamiento; <em>por omisin es nulo</em>.
@param <b>pr:</b> precisin; <em>por omisin es cero</em>.
*/
template <typename T> inline
void LISTA( const T & K, std::string st = "", short pr = 0 ) {
    std::ios::fmtflags oldFlags = std::cout.flags();
    std::cout << st << std::fixed << std::setprecision( pr );

    for ( auto x : K )
        std::cout << x << ", ";

    std::cout << "\b\b ";
    std::cout.flags( oldFlags );
}

/**
@brief MUESTRA: imprime opcionalmente una cadena C++ seguida
por todos los elementos del contenedor K, puestos en una lnea,
separndolos por tabulaciones. Galliano, 2019,
@param <b>K:</b> contenedor del tipo T, que itere hacia adelante.
@param <b>st:</b> encabezamiento; <em>por omisin es nulo</em>.
*/
template <typename T> inline
void MUESTRA( const T & K, std::string st = "" ) {
    std::ios::fmtflags oldFlags = std::cout.flags();
    std::cout << st;

    for ( auto x : K )
        std::cout << '\t' << x;

    std::cout << '\t';
    std::cout.flags( oldFlags );
}

/**
@brief TABULA: imprime opcionalmente una cadena st seguida por todos
los elementos del contenedor K, en una tabla de cc columnas, con pr
decimales de precisin y mostrando o no el signo. Galliano, 2019.
@param <b>K:</b> contenedor del tipo T, que itere hacia adelante.
@param <b>st:</b> encabezamiento; <em>por omisin es nulo.</em>
@param <b>pr:</b> precisin; <em>por omisin es cero.</em>
@param <b>sg:</b> muestra el signo positivo; <em>por omisin no se ve.</em>.
@param <b>cc:</b> cantidad de columnas a consola; <em>por omisin son 10.</em>.
*/
template <typename T> inline
void TABULA( const T & K, std::string st = "", short pr = 0, bool sg = false, short cc = 10 ) {
    short col = 0;
    std::ios::fmtflags oldFlags = std::cout.flags();
    std::cout << st << std::fixed << std::setprecision( pr );
    sg ? std::cout << std::showpos : std::cout << std::noshowpos;

    for ( auto x : K ) {
        ++col;
        std::cout << std::setw( 6 ) << x;
        0 == col % cc ? std::cout << '\n' : std::cout << '\t' ;
    } // for

    std::cout << '\n';
    std::cout.flags( oldFlags );
}

#endif // PRINT_H_INCLUDED
