#ifndef RND_H_INCLUDED
#define RND_H_INCLUDED

//  rnd.h: declaraciones para la generación de números aleatorios
//  Fecha: 2/5/2020; 11:50 pm
//  Por S. Galliano.

#include <algorithm>
#include <functional>

/**
@brief RELLENA_I: llena un contenedor de tamaño dado con valores enteros
aleatorios uniformemente distribuidos en el rango [p:q]. Galliano, 2019
@param <b>C:</b> un contenedor con tamaño dado.
@param <b>p:</b> es el valor entero menor.
@param <b>q:</b> es el valor entero mayor.
*/
template <typename T> inline
void RELLENA_I( T & C, int p, int q ) {
    std::mt19937 mt( rand() );
    std::uniform_int_distribution<int>dist( p, q );
    auto g = bind( dist, mt );
    generate( C.begin(), C.end(), g );
}

/**
@brief RELLENA_R: llena un contenedor de tamaño dado con valores reales
aleatorios uniformemente distribuidos en el rango [p:q). Galliano, 2019
@param <b>C:</b> un contenedor con tamaño dado.
@param <b>p:</b> es el valor menor: float o double.
@param <b>q:</b> es el valor mayor: float o double.
*/
template <typename T> inline
void RELLENA_R(T &C, int p, int q) {
    std::mt19937 mt( rand() );
    std::uniform_real_distribution<double>dist( p, q );
    auto g = bind( dist, mt );
    generate( C.begin(), C.end(), g );
}

/**
@brief GAUSS: llena un contenedor de tamaño dado con valores
reales aleatorios gaussianos normalizados Z~(0, 1) Galliano, 2019
@param <b>C:</b> un contenedor con tamaño dado.
*/
template <typename T> inline
void GAUSS( T & C ) {
    std::mt19937 mt( rand() );
    std::normal_distribution<double>dist( 0, 1 );
    auto g = bind( dist, mt );
    generate( C.begin(), C.end(), g );
}

#endif // RND_H_INCLUDED
