//  cartas.cpp: definiciones para el programa 12
//  Fecha: 25/8/2019; 7:10 am
//  Revisin: 29/3/2020; 6 pm
//  Por S. Galliano.

#include "cartas.h"

//  las figuras de la baraja
enum { Oros, Copas, Espadas, Bastos };

Str ponerPalo( int palo ) {
    if ( Oros    == palo ) return "oros";
    if ( Copas   == palo ) return "copas";
    if ( Espadas == palo ) return "espadas";
    return "bastos";
}

//  los nmeros de la baraja
enum { As, Dos, Tres, Cuatro, Cinco, Seis, Siete, Sota, Caballo, Rey };

Str ponerValor( int valor ) {
    if ( As      == valor ) return "As de ";
    if ( Dos     == valor ) return "Dos de ";
    if ( Tres    == valor ) return "Tres de ";
    if ( Cuatro  == valor ) return "Cuatro de ";
    if ( Cinco   == valor ) return "Cinco de ";
    if ( Seis    == valor ) return "Seis de ";
    if ( Siete   == valor ) return "Siete de ";
    if ( Sota    == valor ) return "Sota de ";
    if ( Caballo == valor ) return "Caballo de ";
    return "Rey de ";
}

#include <algorithm>
#include <ctime>

//  carga y baraja un mazo de cartas
void cargar( Mazo & cartas ) {
    int i = 0;

    do {
        for ( int fig = Oros; fig <= Bastos; ++fig ) {
            for ( int val = As; val <= Rey; ++val ) {
                cartas[i] = ( ponerValor( val ) + ponerPalo( fig ) );
                ++i;
            } // for As hasta Rey
        } // for Oros hasta Bastos
    } while ( i < MAZO ); // do-while

    srand( time( nullptr ) ); // garantiza distintas secuencias por ejecucin
    std::random_shuffle( cartas.begin(), cartas.end() ); // baraja el arreglo
}

#include <iostream>
#include <iomanip>

//  funcin auxiliar: pone una lnea divisoria
void ponerLinea() {
    // CAMPO x PALOS + un blanco entre cada 2 jugadores
    int ancho = CAMPO * PALOS + 3;
    std::cout << std::left
    << std::setw( ancho ) << std::setfill( '-' ) << '\n'
    << std::setfill( ' ' ) << '\n';
}

//  muestra las barajas desde for_each
void mostrar( const Str & carta ) {
    static int j = 0; // lleva la cuenta entre llamadas
    std::cout << std::setw( CAMPO ) << carta;                      // pone la carta
    if ( j == MAZO - 1 ) return;                                   // ltima lnea?
    0 == ( j + 1 ) % PALOS ? std::cout << '\n' : std::cout << ' '; // reparte
    ++j;
}

//  reparte entre 4 jugadores
void repartir( Mazo & cartas ) {
    //  encabezado para 4 jugadores
    for ( int i = 0; i < 4; ++i )
        std::cout << "Jugador " << i + 1 << std::setw( CAMPO );

    //  tabla
    ponerLinea();
    for_each( cartas.begin(), cartas.end(), mostrar );
    ponerLinea();
}
