// Programa 13: ordenamiento
// Fecha: 25/8/2019; 12:47 m
// Revisin: 11/9/2020; 9:11 am
// S. Galliano.

#include "../deposito/rnd.h"
#include "auxiliar.h"
#include <iostream>
#include <iomanip>
#include <algorithm>

const int PASE = 5;

// funcin de comparacin de enteros para qsort()
int comparar( const void * m, const void * n ) {
    int p = *static_cast<const int *>( m );
    int q = *static_cast<const int *>( n );
    return p - q;
}

int main() {
    int m = 1, M = N * 0.5; //  lmites de los nmeros generados

    std::cout <<
    "Programa 13: comparativa de ordenamiento\n\n"
    "Comparaci\242n entre qsort() y las formas de ordenamiento\n"
    "total de la STL, haciendo " << PASE << " pases sobre " << N <<
    " enteros,\nentre " << m << " y " << M <<  ", generados al azar.\n\n"
    "Tenga un poco de paciencia. Trabajando...\n\n";

    // pedido de memoria
    vInt vc1( N ), vc2( N ); // dos vectores
    lInt lst;                // una lista
    sInt mms;                // una bolsa
    aInt ary;                // un arreglo esttico STL
    int * pAry = new int[N]; // un arreglo en la tienda libre

    // las marcas del tiempo y los totales
    clock_t ini, fin, totalIni, totalFin;
    double total_R  = 0, total_qs = 0,
           total_st = 0, total_ss = 0, total_ps = 0,
           total_ls = 0, total_ms = 0, total_hs = 0,
           total_ar = 0, tiempo_total = 0;

    totalIni = clock();

    for ( int i = 0; i < PASE; ++i ) {
        std::cout << "Pase #" << i + 1 << '\r';

        // el tiempo de llenar nuevos valores
        ini = clock();
        RELLENA_I( vc1, m, M );                      // un juego nuevo de valores
        vc2 = vc1;                                   // se pasan al otro vector
        lst.assign( vc1.begin(), vc1.end() );        // se pasan a la lista
        copy( vc1.begin(), vc1.end(), ary.begin() ); // se pasan al arreglo esttico
        copy( vc1.begin(), vc1.end(), pAry );        // se pasan al arreglo dinmico
        fin = clock();
        total_R += difftime( fin, ini ) * 0.001;

        // el tiempo de qsort() sobre un vector en la tienda libre
        ini = clock();
        qsort( pAry, N, sizeof( int ), cmpia );
        fin = clock();
        total_qs += difftime( fin, ini ) * 0.001;

        // el tiempo de sort() sobre un arreglo STL
        ini = clock();
        std::sort( ary.begin(), ary.end() );
        fin = clock();
        total_ar += difftime( fin, ini ) * 0.001;

        // el tiempo de sort() sobre un vector STL
        ini = clock();
        std::sort( vc2.begin(), vc2.end() );
        fin = clock();
        total_st += difftime( fin, ini ) * 0.001;
        vc2 = vc1; // vc2 est azarosa de nuevo

        // el tiempo de stable_sort() sobre un vector STL
        ini = clock();
        stable_sort( vc2.begin(), vc2.end() );
        fin = clock();
        total_ss += difftime( fin, ini ) * 0.001;
        vc2 = vc1; // vc2 est azarosa de nuevo

        // el tiempo de sort_heap()
        ini = clock();
        make_heap( vc2.begin(), vc2.end() ); // convierte la secuencia en rbol
        sort_heap( vc2.begin(), vc2.end() ); // convierte el rbol en secuencia
        fin = clock();
        total_hs += difftime( fin, ini ) * 0.001;
        vc2 = vc1; // vc2 est azarosa de nuevo

        // el tiempo de partial_sort() sobre un vector STL
        ini = clock();
        partial_sort( vc2.begin(), vc2.end(), vc2.end() );
        fin = clock();
        total_ps += difftime( fin, ini ) * 0.001;
        vc2 = vc1; // vc2 est azarosa de nuevo

        // el tiempo de la lista
        ini = clock();
        lst.sort();
        fin = clock();
        total_ls += difftime( fin, ini ) * 0.001;

        // el tiempo del multiset sobre una lista azarosa
        ini = clock();
        mms.clear();
        mms.insert( vc2.begin(), vc2.end() );
        fin = clock();
        total_ms += difftime( fin, ini ) * 0.001;
    } // for

    totalFin = clock();
    tiempo_total = difftime( totalFin, totalIni ) * 0.001;

    // la salida
    std::cout << std::fixed << std::setprecision( 4 ) <<
    "\n\nTiempo total consumido: " << tiempo_total << " s" <<
    "\nTiempo promedio consumido por:\n"
    "  genera valores = " << total_R  / PASE << " s/pase.\n"
    "  qsort()        = " << total_qs / PASE << " s/pase.\n"
    "  array: sort()  = " << total_ar / PASE << " s/pase.\n"
    "  lista: sort()  = " << total_st / PASE << " s/pase.\n"
    "  stable_sort()  = " << total_ss / PASE << " s/pase.\n"
    "  sort_heap()    = " << total_hs / PASE << " s/pase.\n"
    "  partial_sort() = " << total_ps / PASE << " s/pase.\n"
    "  list.sort()    = " << total_ls / PASE << " s/pase.\n"
    "  multiset       = " << total_ms / PASE << " s/pase.\n\n";

    system( "pause" );
    delete [] pAry;
    return EXIT_SUCCESS;
}
