//  entrada.cpp: definiciones para el programa 16
//  Fecha: 26/8/2019; 7:52 am
//  Revisin: 11/9/2020; 12:22 pm
//  Por S. Galliano.

#include "entrada.h"
#include <iostream>
#include <fstream>

//  valores ledos desde el archivo
void Estads::leerArchivo( const Str archy ) {
    // aqu hace falta cdigo defensivo, porque
    // si el fichero no est, no puede abrirlo
    std::ifstream Leyendo;
    Leyendo.open( archy );

    if ( Leyendo.is_open() ) {
        double val;

        while ( Leyendo >> val )
            vec.push_back( val );

        sort( vec.begin(), vec.end() );
    } else {
        throw ERROR;
    } // if-else
}

double Estads::suma()     { return accumulate(vec.begin(), vec.end(), 0.0); }
double Estads::promedio() { return suma() / vec.size(); }
double Estads::miMax()    { return *max_element(vec.begin(), vec.end()); }
double Estads::miMin()    { return *min_element(vec.begin(), vec.end()); }

// mediana
double Estads::mediana() {
    double med;
    int n = vec.size();

    int idx = 0.5 * n; // se busc el ndice del medio
    n % 2              // se calcul la mediana
        ? med = vec[idx]
        : med = 0.5 * ( vec[idx - 1] + vec[idx] );
    return med;
}

// moda
double Estads::moda() {
    double laModa, unaModa;
    int laRacha, unaRacha;
    laModa = unaModa = vec[0];
    laRacha = unaRacha = 0;
    unsigned i = 0;          // el ndice de la secuencia
    unsigned n = vec.size(); // el tamao

    do {
        if ( unaModa not_eq vec[i] ) {  // cambi la moda?
            unaModa = vec[i];           // SI: actualizarla

            if ( unaRacha > laRacha ) { // cambi la racha?
                laRacha = unaRacha;     // SI: actualizarla y...
                laModa  = vec[i - 1];   // ...actualizar la moda
            } // if-interior

            unaRacha = 0; // contar otra racha
        } // if-exterior

        ++unaRacha; // incrementa la racha
        ++i;        // incrementa el ndice
    } while ( i not_eq n ); // do-while

    if ( 1 == laRacha )
        laModa = static_cast<double>( HUGE_VAL );

    return laModa;
}

// varianza muestral
double Estads::varianza() {
    std::vector<double>aux( vec );
    int n = vec.size();
    transform( aux.begin(), aux.end(),    // el rango a multiplicar
               aux.begin(),               // la 2da. entrada
               aux.begin(),               // a partir de aqu
               std::multiplies<double>()  // multiplica 1ra. por 2da.
             );
    double SDC = accumulate( aux.begin(), aux.end(), 0.0 ); // suma de cuadrados
    return (SDC - suma() * suma() / n) / ( n - 1 ) ;
}

#include <iomanip>
#include <ios>

// la escritura
void muestra( std::ostream & escribe, Str ID, double stat ) {
    using std::cout;
    std::ios::fmtflags oldFlags = escribe.flags();
    escribe.setf( std::ios::fixed );
    escribe << std::setprecision( 4 );

    if ( "Cuenta = " == ID ) {
        escribe << std::setprecision( 0 );
        escribe << std::setw( 12 ) << ID << std::setw( 4 ) << stat << '\n';
    } else if ( HUGE_VAL == stat ) {
        escribe << "     " + ID + " #N/A" << '\n';
    } else {
        escribe << std::setw( 12 ) << ID << std::setw( 9 ) << stat << '\n';
    } // if-else

    escribe.unsetf( std::ios::fixed );
    escribe.flags(oldFlags);
}
