//  cpa.cpp: definiciones para el programa 20
//  Fecha: 5/5/2020; 8:15 am
//  Revisin: 12/9/2020; 10:15 am
//  Por: S. Galliano.
#include "cpa.h"

//  los obreros
//  entra las variables a la tupla
void Obrero::entrarObrero( Str nombre, Str apellido, int horas ) {
    dato = std::make_tuple( nombre, apellido, horas );
}

//  devuelve el dato
Datos Obrero::sacarDatos() { return dato; }

//  verifica si un dato ya est en la nmina;
//  slo toma las variables necesarias
bool Obrero::yaEsta( Datos unDato ) {
    Str nombre     = std::get<0>( dato );
    Str apellido   = std::get<1>( dato );
    Str unNombre   = std::get<0>( unDato );
    Str unApellido = std::get<1>( unDato );
    return ( nombre == unNombre and apellido == unApellido );
}

//  la CPA
//  checa si el dato ya est
bool CPA::datoEsta( Par dat ) const {
    Obrero tmp;
    tmp.entrarObrero( dat.first, dat.second, 0 );
    for ( auto x : nomina )
        if ( x.yaEsta( tmp.dato ) )
            return true;

    return false;
}

#include "extras.h"
#include <ctime>
#include <cstdlib>
#include <iostream>
#include <algorithm>
#include <iomanip>
using std::setw;
using std::ostream;

//  calcula el salario de un obrero y pone la lnea en la nmina
double CPA::verDatos( Datos & d, ostream & Escribiendo ) const {
    Str nom, ape;
    int hs;
    std::tie( nom, ape, hs ) = d;
    double sal = calcSalario( hs );
    Escribiendo << std::setfill( '.' ) << std::fixed  << std::left <<
                setw ( 20 ) << ( ape + ", " + nom ) << std::right <<
                setw ( 5 )  << hs  << std::setprecision( 2 ) <<
                setw ( 15 ) << sal << "  _______________\n";
    return sal;
}

//  escribe la nmina a un flujo de salida;
//  llama a cada obrero de la nmina
void CPA::escribirNomina( ostream & Escribiendo ) {
    if ( Escribiendo.fail() ) throw ERR_E;

    sort( nomina.begin(), nomina.end(), menor );
    double total = 0;                 // total nmina
    time_t laFecha = time( nullptr ); // obteniendo la fecha completa
    Str fecha = ctime( &laFecha );    // la fecha y la hora
    //  encabezado
    system( "cls" );
    Escribiendo <<
    "CPA S. Garc\241a - N\242mina" << '\n' << "Fecha: " << fecha << '\n' <<
    std::left << setw( 18 ) << "Obrero" << std::right << setw( 7 ) << "Horas" <<
    setw ( 16 ) << "Salario\n";
    //  visitar la nmina
    Obrero tmp;
    Dek::const_iterator i;
    Dek::const_iterator end( nomina.end() );

    for ( i = nomina.begin(); i != end; ++i ) {
        tmp = *i;
        total += verDatos( tmp.dato, Escribiendo ); // a consola o a disco
    } // for

    Escribiendo
    << std::left  << setw( 30 ) << "Total"
    << std::right << setw( 10 ) << total << "$ Firma\n\n";
}

using std::cerr;
using std::cout;

void CPA::borrarObrero( Par dat ) {
    if ( datoEsta( dat ) ) {
        //  visitar la nmina
        Obrero tmp, proxy;
        tmp.entrarObrero( dat.first, dat.second, 0 );
        Dek::iterator i;
        Dek::iterator end( nomina.end() );

        for ( i = nomina.begin(); i != end; ++i ) {
            proxy = *i; // bajar un nivel de indireccin

            if ( proxy.yaEsta( tmp.dato ) ) {
                nomina.erase( i );
                cout << HECHO;
                return;
            } // if
        } // for
    } else {
        cerr << MSG_N;
    } // if-else
}

using std::cin;

//  aade un obrero a la nmina
void CPA::adicionarObrero( Par dat ) {
    if ( datoEsta( dat ) )
        cerr << MSG_Y;
    else {
        Obrero tmp;
        int hs;
        cout << "\250Horas? ";
        ( cin >> hs ).get();
        tmp.dato = std::make_tuple( dat.first, dat.second, hs );
        nomina.push_back( tmp );
        cout << HECHO;
    } // if-else
}

void CPA::editarObrero( Par dat ) {
    if ( datoEsta( dat ) ) {
        //  visitar la nmina
        Obrero tmp, proxy;
        tmp.entrarObrero( dat.first, dat.second, 0 );
        Dek::iterator i;
        Dek::iterator end( nomina.end() );

        for ( i = nomina.begin(); i != end; ++i ) {
            proxy = *i; // bajar un nivel de indireccin

            if ( proxy.yaEsta( tmp.dato ) ) {
                int hs;
                cout << "\250Nuevas horas? ";
                ( cin >> hs ).get();
                nomina.erase( i );
                tmp.dato = std::make_tuple( dat.first, dat.second, hs );
                nomina.push_back( tmp );
                cout << HECHO;
                return;
            } // if
        } // for
    } else {
        cerr << MSG_N;
    } // if-else
}

#include <cstring>
#include <fstream>
using std::getline;

void CPA::leerPlantilla( std::istream & Leyendo ) {
    if ( Leyendo.fail() ) throw ERR_L;

    Obrero tmp;
    Str nom, ape, hs;

    while ( Leyendo.good() ) {
        //  construyendo un registro
        getline( Leyendo, nom, ',' );
        getline( Leyendo, ape, ',' );
        getline( Leyendo, hs, '\n' );
        tmp.entrarObrero( nom, ape, atoi( hs.c_str() ) );
        nomina.push_back( tmp );
    } // while
}
